<?php

namespace wcf\system\cache\builder;

use wcf\system\cache\CacheHandler;
use wcf\system\exception\SystemException;
use wcf\system\WCF;
use wcf\util;
use wcf\util\DateUtil;
use wcf\util\HTTPRequest;
use wcf\util\JSON;
/** Cache handler for my.eListing plugin
 *
 * @author      PCSG (Jan)
 * @package     de.pcsg.wcf.myelisting
 * @subpackage  system.cache.source
 * @category    Community Framework
 */
class MyElistingCacheBuilder extends AbstractCacheBuilder
{
    /**
     * @see	\wcf\system\cache\builder\AbstractCacheBuilder::$maxLifetime
     */
    protected $maxLifetime = 600;


    /**
     * @var string URL to my.eListing API. Can be changed in options.xml
     */
    private $api = 'https://'.MYELISTING_URL.'/api';


    /**
     * @param array $parameters
     * @return array
     * @throws \wcf\system\database\DatabaseException
     */
    public function rebuild(array $parameters) {

        if(isset($parameters['function']) && isset($parameters['value'])) {

            if($parameters['function'] == 'apiAvailable') {

                $headers = @get_headers($this->api);

                $apiAvailable = false;

                if($headers !== false && strpos($headers[0],'404') === false) {
                    $apiAvailable = true;
                }

                $this->store("apiAvailable", 0, $apiAvailable);

            }


            if($parameters['function'] == 'getItemById') {

                $itemID = $parameters['value'];

                $request = new HTTPRequest($this->api . "/getItemById/" . $itemID);
                $request->execute();

                $apiReturn = $request->getReply()['body'];

                // If my.eListing API not available set number articles to 'Error'
                if ($apiReturn === false) {
                    $itemInfo['error'] = 'apiUnavailable';
                } else {
                    $apiReturn = JSON::decode($apiReturn, true);

                    if(isset($apiReturn['Timestamp'])) {
                        $itemInfo['timestamp'] = $apiReturn['Timestamp'];
                    } else {
                        $itemInfo['timestamp'] = '';
                    }

                    if(isset($apiReturn['Item']['GalleryURL'])) {
                        $itemInfo['thumbnail'] = $apiReturn['Item']['GalleryURL'];
                    } else {
                        $itemInfo['thumbnail'] = 'wcf/images/myelisting/ebay_logo.png';
                    }

                    if(isset($apiReturn['Item']['Title'])) {
                        $itemInfo['title'] = $apiReturn['Item']['Title'];
                    } else {
                        $itemInfo['title'] = '';
                    }

                    if(isset($apiReturn['Item']['ConvertedCurrentPrice']['Value'])) {
                        $itemInfo['price'] = $apiReturn['Item']['ConvertedCurrentPrice']['Value'];
                    } else {
                        $itemInfo['price'] = '';
                    }

                    if(isset($apiReturn['Item']['ConvertedCurrentPrice']['CurrencyID'])) {
                        $itemInfo['currency'] = $apiReturn['Item']['ConvertedCurrentPrice']['CurrencyID'];
                    } else {
                        $itemInfo['currency'] = '';
                    }

                    if(isset($apiReturn['Item']['TimeLeft'])) {
                        $itemInfo['timeLeft'] = $apiReturn['Item']['TimeLeft'];
                    } else {
                        $itemInfo['timeLeft'] = '';
                    }

                    $this->store("getItemById", $itemID, $itemInfo);
                }

            }


            if($parameters['function'] == 'numberArticles') {

                $ebayName = $parameters['value'];

                $request = new HTTPRequest($this->api . "/getNumberItemsBySeller/" . $ebayName);
                $request->execute();

                $apiReturn = $request->getReply()['body'];

                // If my.eListing API not available set number articles to 'Error' and cache for only 5 min
                if ($apiReturn === false) {
                    $numberArticles = 'Error';
                    $this->store("numberArticles", $ebayName, $numberArticles);
                } else {
                    $numberArticles = JSON::decode($apiReturn, true);
                    $this->store("numberArticles", $ebayName, $numberArticles);
                }
            }

        }



        $sql = "SELECT	cache_category, cache_key, cache_value
			    FROM	wcf".WCF_N."_myelisting_cache;";

        $db = WCF::getDB();
        $statement = $db->prepareStatement($sql);
        $statement->execute();

        $data = array( array() ); // Keep this for loop beneath
        while ($row = $statement->fetchArray()) {

            $category = $row['cache_category'];
            $key      = $row['cache_key'];
            $value    = $row['cache_value'];

            $data[$category][$key] = $value;
        }

        // e.g: $data['numberArticles']['username'] = 60
        // Meaning: User 'username' has '60' articles
        return $data;

    }

    public function store($category,$index,$value) {

        $value = util\JSON::encode($value, true);

        if($this->keyExists($category, $index)) {
            $sql = 'UPDATE wcf'.WCF_N.'_myelisting_cache
                    SET cache_value = ?
                    WHERE cache_category = ? AND cache_key = ?
                    LIMIT 1';
        } else {
            $sql = 'INSERT INTO wcf'.WCF_N.'_myelisting_cache (cache_value, cache_category, cache_key)
                    VALUES (?, ?, ?)';
        }


        $db = WCF::getDB();
        $statement = $db->prepareStatement($sql);
        try {
            $statement->execute([$value, $category, $index]);
        } catch(\Exception $e) {}

        $this->reset();
    }


    public function keyExists($category, $key) {
        try {
            $dbCategory = $this->getData(array(), $category);
            if(isset($dbCategory[$key])) {
                return true;
            }
        } catch(\Exception $e) { }

        return false;
    }




}